/*************************************************************************/
/*
//  Airline Crew Scheduling With GA
//
//
*/
/*************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <new.h>

#include "AirFileReader1.h"
#include "AirScheduleProblem.h"
#include "AirScheduleSolu2.h"
#include "AirFitnessFunction.h"
#include "AirInitializer.h"
#include "AirMutation.h"
//#include "AirCrossover.h"
//#include "AirCrossover2.h"
#include "City.h"
#include "EdgeGraph.h"
#include "../deffunc.h"

#include <Sort_Recombinator.h>
#include <Stat_Recombinator.h>
#include <Generational_Recom.h>

#include <SimilarParentSelector.h>
#include <GeneticAlgorithm.h>

typedef long time_t;
typedef long clock_t;

/*
// Memory Allocation Error Handler.
// It will be called when there is no Enough 
// Memory.
*/


void allocation_handler()
{
   cout << "Out of Memory. Aborting Program...."<< endl;
   exit(1);
}

/*
//
*/

void solvewithGASTD(AirScheduleProblem *AirInstance)
{
   AirFitnessFunction   fitFunc;
   AirInitializer       Initme(AirInstance);
   AirMutation          Jmutation;
   //AirCrossover         Jcross;
   //AirCrossover2         Jcross;
   Population       myPop(5);    

   Initme.apply(myPop);

   Sort_Recombinator     SRecombinator;
   SimilarParentSelector SimilarSelect;
    
   GeneticAlgorithm mySolver(&fitFunc,NULL,&Jmutation,&SimilarSelect,&SRecombinator,&Initme);

   int MaxGenNum= 2;
   int PopulationSize = 5;
   float MutationProbability = 0.2;
   float CrossoverProbability = 0.6;
   int TestTimes = 1;
   int TestType = 0;

   mySolver.SetMaxGeneration(MaxGenNum);
   mySolver.SetPopulationSize(PopulationSize);
   mySolver.SetMutationProb(MutationProbability);
   mySolver.SetCrossProb(CrossoverProbability);
   //mySolver.run();
   if(TestType>0&&TestType<4) mySolver.setTestType(TestType);

   cout << " before run() " << endl;
   /*
   //mySolver.runFuzzyTimes(maximum(1,TestTimes));
   mySolver.run();
   cout << " after run() " << endl;
   */

   cout << " Current ppopulation " << myPop << endl << "0000" << endl;
   Population *nPop = mySolver.runOnceOn(myPop);
   cout << " After one epoch " << *nPop << endl;

   return;

}// end of solvewithGASTD()

/*
void solvewithGA(AirScheduleProblem AirInstance)
{
   AirFitnessFunction   fitFunc;
   AirInitializer       Initme(&AirInstance);
   AirMutation          Jmutation;
   AirCrossover         Jcross;

   Recombinator         *SRecombinator;
   SimilarParentSelector SimilarSelect;
    
   cout << " I defined everything " << endl;

   int START = 0, noRun=1;
   int MaxGenNum;
   int PopulationSize;
   float MutationProbability;
   float CrossoverProbability;
   int TestTimes;
   int TestType;
   int PopStrategy;
   char Answer[4];
   int  isValid;


   while(!START)
   {  
      cout << " Enter the Generation Number : ";
      cin  >> setw(4) >> MaxGenNum;

      cout << " Enter the Population Size : ";
      cin  >> setw(4) >> PopulationSize;

      cout << " Enter the Mutation Probability : ";
      cin  >> setw(4) >> MutationProbability;

      cout << " Enter the Crossover Probability : ";
      cin  >> setw(9) >> CrossoverProbability;

      cout << " Enter the number of Try : ";
      cin  >> setw(4) >> TestTimes;

      cout << " Enter the Test Type (0:standart, 1:Cross, 2:Mutation, 3: Cross&Mutation) : ";
      cin  >> setw(2) >> TestType;

      isValid=0;

      while(!isValid)
      {  cout << " Enter the population generarion strategy (0:Steady, 1:Generational, 2:Sorted Bests) :";
         cin >> setw(2) >> PopStrategy;
	 if(PopStrategy>=0 && PopStrategy<3) isValid=1;
	 else cout << " Please enter one of those {0,1,2} "<<endl;

      }

      cout << " Do you want to change anything ?: ";
      cin  >> setw(4) >> Answer;
      if(!strcmp(Answer,"N")||!strcmp(Answer,"No")|| 
	 !strcmp(Answer,"n")||!strcmp(Answer,"no"))
      {  START = 1;
         cout << " Do you want to run the GA ?: ";
         cin  >> setw(4) >> Answer;
         if(!strcmp(Answer,"N")||!strcmp(Answer,"No")||
	    !strcmp(Answer,"n")||!strcmp(Answer,"no")) noRun = 0;
      }
   }// end of While


   // define the population combinator type
   
   switch(PopStrategy)
   {  case 0 : SRecombinator = new Stat_Recombinator(); break;
      case 1 : SRecombinator = new Generational_Recom(); break;
      case 2 : SRecombinator = new Sort_Recombinator(); break;
      default: SRecombinator = new Stat_Recombinator(); break;
   }// end of switch


   GeneticAlgorithm mySolver(&fitFunc,&Jcross,&Jmutation,&SimilarSelect,SRecombinator,&Initme);

   cout << " I defined everything " << endl;

   time_t start;
   time_t stop;
   cout << "\n with GA Time: " << ctime(&start) << endl;

   if(noRun)
   {   
       mySolver.SetMaxGeneration(MaxGenNum);
       mySolver.SetPopulationSize(PopulationSize);
       mySolver.SetMutationProb(MutationProbability);
       mySolver.SetCrossProb(CrossoverProbability);
       if(TestType>0&&TestType<4) mySolver.setTestType(TestType);
       cout << " before run" << endl;
       mySolver.runFuzzyTimes(maximum(1,TestTimes));
       //mySolver.run();

   }// end of if

   cout << "the end Time: " << ctime(&stop) << endl;
   cout << " The process took " << difftime(stop,start) << " secs " << endl;
   cout << " The process took " << start << "-"<< stop << " secs " << endl;

   return;
}
*/

main(int argc, char *argv[])
{  int i; 
   // Set Allocation Exception Handler
   set_new_handler(allocation_handler);
   srand(150);

   AirFileReader1 Reader("data");  
   if(argc>1) Reader.setFileName(argv[1]);
   AirScheduleProblem Prob ;
   Reader.read(Prob);

   unsigned indx = Prob.lookUp("ERCAN");
   City a("ERCAN",indx);
   Prob.addBaseCity(a);

   indx = Prob.lookUp("ISTANBUL");
   City b("ISTANBUL",indx);
   Prob.addBaseCity(b);

   /*indx = Prob.lookUp("IZMIR");
   City c("IZMIR",indx);
   Prob.addBaseCity(c);

   indx = Prob.lookUp("ANKARA");
   City d("ANKARA",indx);
   Prob.addBaseCity(d);

   indx = Prob.lookUp("ADANA");
   City e("ADANA",indx);
   Prob.addBaseCity(e);
   */

   cout << Prob ;
   Prob.setCostPerOvernight(50); 
   Prob.setCostPerMinute(1); 

   //AirScheduleSolu AirSolInstance(&Prob);
   
   /*for(i=0;i<3;i++)
   {  
      AirScheduleSolu AirSolInstance(&Prob);
      AirSolInstance.solveRandom3();
      cout << AirSolInstance ;
   }*/

   cout << "\n with GA " << endl;
   //solvewithGA(Prob);
   //solvewithGASTD(Prob);
   cout << "\n end GA " << endl;

   EdgeGraph eg(&Prob,Prob.getNumberOfFlights());

   eg.buildGraphFor(&Prob);
   Prob.setEdgeGraph(&eg); 

   cout << eg << endl;

   //AirScheduleSolu2 AirSolInstance(&Prob,&eg);
   AirScheduleSolu2 AirSolInstance(&Prob,&eg);

   /*for(i=0;i<10;i++)
   {
      AirScheduleSolu2 AirSolInstance(&Prob,Prob.getEdgeGraph());
      AirSolInstance.solveRandom();

      cout << " Run #  :" << i << endl << AirSolInstance << endl;

   }*/

   cout << "\n with GA " << endl;
   //solvewithGA(Prob);
   solvewithGASTD(&Prob);
   cout << "\n end GA " << endl;

}

